# Created by Q-ays.
# whosqays@gmail.com

# install sqlalchemy before use

# postgres tools

from sqlalchemy.exc import SQLAlchemyError, TimeoutError
import traceback
import json


def session_exception(session, is_raise=True):
    def wrapper(func):

        def catch(*args, **kwargs):
            try:
                result = func(*args, **kwargs)
                return result
            except SQLAlchemyError as e:
                # print(e)
                print('~~~~~~~~~~~~~~~~session error~~~~~~~~~~~~~~~~~~')
                traceback.print_exc()
                if not isinstance(e, TimeoutError):
                    session.rollback()
                if is_raise:
                    raise e

        return catch

    return wrapper


def to_dict(self):
    return {c.name: getattr(self, c.name, None) for c in self.__table__.columns}


def str2js(ls):
    """
    json字符转字典
    :param ls:
    :return:
    """
    if isinstance(ls, list):
        ls0 = list()

        try:
            for l in ls:
                ls0.append(json.loads(l))

            return ls0
        except:
            pass

    return ls


def detect_filed(o, *args):
    """
    特定字段字符转json
    :param o:
    :param args:
    :return:
    """
    for arg in args:
        n = o.get(arg)
        if n:
            o[arg] = str2js(n)

    return o


def o2d(obj):
    """
    把对象(支持单个对象、list、set)转换成字典
    针对postgres数据库
    :param obj: obj, list, set
    :return:
    """

    if isinstance(obj, dict) or (not obj):
        return obj

    is_list = isinstance(obj, list)
    is_set = isinstance(obj, set)

    if is_list or is_set:
        obj_arr = []

        for o in obj:
            if o:
                if isinstance(o, dict):
                    n = detect_filed(o, 'roles', 'org')
                else:
                    n = detect_filed(o.to_dict(), 'roles', 'org')

                obj_arr.append(n)
        return obj_arr
    else:
        return detect_filed(obj.to_dict(), 'roles', 'org')


def repo_ref(session0):
    """
    基于postgres 数据库增删改查的公共类
    :param session0:
    :return:
    """
    class RepoBase:
        def __init__(self, Model=None):
            self.session = session0
            self.Model = Model

        def add(self, **data):
            model0 = self.Model(**data)

            self.session.add(model0)
            self.session.commit()

            return model0

        def delete(self, did):
            repo = self.session.query(self.Model).get(did)

            self.session.delete(repo)
            self.session.commit()

            return repo

        def update(self, did, **data):
            model0 = self.get(did)

            columns = model0.__table__.columns

            for col in columns:
                name = col.name
                value = data.get(name, None)

                if value and not col.primary_key:
                    setattr(model0, name, value)

            self.session.commit()

            return model0

        def get(self, did=None, **kwargs):
            if did:
                repos = self.session.query(self.Model).get(did)
            elif kwargs:
                repos = self.session.query(self.Model).filter_by(**kwargs).all()
            else:
                repos = self.session.query(self.Model).all()

            return repos

    return RepoBase



'''
第二种方法：如果模块是以class组织代码的
```
from wisdom.example_class import my_class
mc=my_class()
mc.hello()
mc.world()
```
'''
import logging


class ExampleClass:
    name = 'sky is blue'
    def __init__(self):
        self.age = 10

    def my_hello(self):
        print('i am a object method')

    def hello(self):
        print('i am a object method')

    @staticmethod
    def hello():
        print('hello{}'.format('world,staticmethod'))

    @classmethod
    def world(cls):
        print('{},world'.format('hello'))

    @classmethod
    def hello(cls):
        print('hello{}'.format('world,classmethod'))





# Install nameko before use

"""
    Example::

        from wisdoms.auth import permit

        host = {'AMQP_URI': "amqp://guest:guest@localhost"}

        auth = permit(host)

        class A:
            @auth
            def func():
                pass
"""

from nameko.standalone.rpc import ClusterRpcProxy
from functools import wraps


def permit(host):
    """ 调用微服务功能之前，进入基础微服务进行权限验证

    :param: host: micro service host
    """

    def wrapper(f):
        @wraps(f)
        def inner(*args, **kwargs):
            service = args[0].name
            func = f.__name__
            token = args[1].get('token')

            with ClusterRpcProxy(host) as r:
                res = r.baseUserApp.verify({'service': service, 'func': func, 'token': token})
            if res:
                args[1]['user'] = res
                return f(*args, **kwargs)

            raise Exception('verified failed')

        return inner

    return wrapper


def add_uid(host):
    """
    用户token 返回用户id
    :param host:
    :return:
    """

    def wrapper(f):
        @wraps(f)
        def inner(*args, **kwargs):
            token = args[1].get('token')

            with ClusterRpcProxy(host) as r:
                res = r.baseUserApp.get_uid({'token': token})
            if res:
                args[1]['uid'] = res
                return f(*args, **kwargs)

            raise Exception('verified failed')

        return inner

    return wrapper


def add_user(host):
    """
    用户token 返回用户信息
    :param host:
    :return:
    """

    def wrapper(f):
        @wraps(f)
        def inner(*args, **kwargs):
            token = args[1].get('token')

            with ClusterRpcProxy(host) as r:
                res = r.baseUserApp.get_user({'token': token})
            if res:
                args[1]['user'] = res
                return f(*args, **kwargs)

            raise Exception('verified failed')

        return inner

    return wrapper



# Created by Q-ays.
# whosqays@gmail.com


"""
    状态码

    Notice::
        大于0，顺利执行
        小于0，执行出错
        大于100，执行成功，出现警告

    Example::
        from wisdoms.commons import revert, codes

        def foo():
            # return revert(1)
            return revert(codes.SUCCESS)


"""

ERROR = -1
MS_ERROR = -2
DATABASE_ERROR = -3
NOT_AUTHORIZED = -4
DATA_FORMAT_ERROR = -5
TIMEOUT = -6
UNKNOWN_ERROR = -7
THIRD_PARTY_ERROR = -8
PARAM_ERROR = -9
GET_WAY_ERROR = -10
CONNECTION_TIMEOUT = -11
ES_SERVER_ERROR = -12
VM_ABNORMAL_STATE = -13

SUCCESS = 1

WARNING = 100

code2name = {

    ERROR: 'ERROR',
    MS_ERROR: 'MS_ERROR',
    DATABASE_ERROR: 'DATABASE_ERROR',
    NOT_AUTHORIZED: 'NOT_AUTHORIZED',
    DATA_FORMAT_ERROR: 'DATA_FORMAT_ERROR',
    TIMEOUT: 'TIMEOUT',
    UNKNOWN_ERROR: 'UNKNOWN_ERROR',
    THIRD_PARTY_ERROR: 'THIRD_PARTY_ERROR',
    PARAM_ERROR: 'PARAM_ERROR',
    GET_WAY_ERROR: 'GET_WAY_ERROR',
    CONNECTION_TIMEOUT: 'CONNECTION_TIMEOUT',
    ES_SERVER_ERROR: 'ES_SERVER_ERROR',
    VM_ABNORMAL_STATE: 'VM_ABNORMAL_STATE',

    SUCCESS: 'SUCCESS',

    WARNING: 'WARNING'
}

code_desc = {
    ERROR: '错误',
    MS_ERROR: '微服务错误',
    DATABASE_ERROR: '数据库错误',
    NOT_AUTHORIZED: '未授权，拒绝访问',
    DATA_FORMAT_ERROR: '数据格式错误',
    TIMEOUT: '超时',
    UNKNOWN_ERROR: '未知错误',
    THIRD_PARTY_ERROR: '第三方服务错误',
    PARAM_ERROR: '参数错误',
    GET_WAY_ERROR: 'api 网关服务器错误',
    ES_SERVER_ERROR: 'elastic search 服务器错误',
    CONNECTION_TIMEOUT: '连接超时',
    VM_ABNORMAL_STATE: '虚机非正常状态',

    SUCCESS: '成功',

    WARNING: '警告'
}

name2code = {
    'ERROR': ERROR,
    'MS_ERROR': MS_ERROR,
    'DATABASE_ERROR': DATABASE_ERROR,
    'NOT_AUTHORIZED': NOT_AUTHORIZED,
    'DATA_FORMAT_ERROR': DATA_FORMAT_ERROR,
    'TIMEOUT': TIMEOUT,
    'UNKNOWN_ERROR': UNKNOWN_ERROR,
    'THIRD_PARTY_ERROR': THIRD_PARTY_ERROR,
    'PARAM_ERROR': PARAM_ERROR,
    'GET_WAY_ERROR': GET_WAY_ERROR,
    'CONNECTION_TIMEOUT': CONNECTION_TIMEOUT,
    'ES_SERVER_ERROR': ES_SERVER_ERROR,
    'VM_ABNORMAL_STATE': VM_ABNORMAL_STATE,

    'SUCCESS': SUCCESS,

    'WARNING': WARNING
}

name_desc = {
    'ERROR': '错误',
    'MS_ERROR': '微服务错误',
    'DATABASE_ERROR': '数据库错误',
    'NOT_AUTHORIZED': '未授权，拒绝访问',
    'DATA_FORMAT_ERROR': '数据格式错误',
    'TIMEOUT': '超时',
    'UNKNOWN_ERROR': '未知错误',
    'THIRD_PARTY_ERROR': '第三方服务错误',
    'PARAM_ERROR': '参数错误',
    'GET_WAY_ERROR': 'api 网关服务器错误',
    'CONNECTION_TIMEOUT': '连接超时',
    'ES_SERVER_ERROR': 'elastic search 服务器错误',
    'VM_ABNORMAL_STATE': '虚机非正常状态',

    'SUCCESS': '成功',

    'WARNING': '警告'
}


def revert(code1, data=None, desc=None):
    """
    公共返回方法

    :param code1: 状态码
    :param data: 返回数据
    :param desc: 描述
    :return: dict
    """

    try:
        msg = code2name[code1]

        if not desc:
            desc = code_desc[code1]
    except KeyError as e:
        return revert(-9, e, '调用公共返回方法出错，未找到状态码，请检查状态码是否匹配')

    if data is not None:
        return {'code': code1, 'msg': msg, 'desc': desc, 'data': data}
    else:
        return {'code': code1, 'msg': msg, 'desc': desc}
